from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.message_content_type_1_item import MessageContentType1Item
from ..models.message_params import MessageParams
from ..models.message_role_enum import MessageRoleEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="Message")


class MessageDict(TypedDict):
    role: MessageRoleEnum
    content: NotRequired[Union[List["MessageContentType1Item"], Unset, str]]
    template_id: NotRequired[Union[Unset, str]]
    params: NotRequired[Union[Unset, MessageParams]]
    pass


@_attrs_define
class Message:
    """
    Attributes:
        role (MessageRoleEnum): * `user` - user
            * `assistant` - assistant
        content (Union[List['MessageContentType1Item'], Unset, str]): The content of the message.
        template_id (Union[Unset, str]): The ID of the template to use.
        params (Union[Unset, MessageParams]): The parameters (key: value) to use with the given template.
    """

    role: MessageRoleEnum
    content: Union[List["MessageContentType1Item"], Unset, str] = UNSET
    template_id: Union[Unset, str] = UNSET
    params: Union[Unset, "MessageParams"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        role = self.role.value

        content: Union[List[Dict[str, Any]], Unset, str]
        if isinstance(self.content, Unset):
            content = UNSET
        elif isinstance(self.content, list):
            content = []
            for content_type_1_item_data in self.content:
                content_type_1_item = content_type_1_item_data.to_dict()
                content.append(content_type_1_item)

        else:
            content = self.content

        template_id = self.template_id

        params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.params, Unset):
            params = self.params.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
            }
        )
        if content is not UNSET:
            field_dict["content"] = content
        if template_id is not UNSET:
            field_dict["template_id"] = template_id
        if params is not UNSET:
            field_dict["params"] = params

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_content_type_1_item import MessageContentType1Item
        from ..models.message_params import MessageParams

        d = src_dict.copy() if src_dict else {}
        role = MessageRoleEnum(d.pop("role"))

        def _parse_content(data: object) -> Union[List["MessageContentType1Item"], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_1 = []
                _content_type_1 = data
                for content_type_1_item_data in _content_type_1:
                    content_type_1_item = MessageContentType1Item.from_dict(content_type_1_item_data)

                    content_type_1.append(content_type_1_item)

                return content_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List["MessageContentType1Item"], Unset, str], data)

        content = _parse_content(d.pop("content", UNSET))

        template_id = d.pop("template_id", UNSET)

        _params = d.pop("params", UNSET)
        params: Union[Unset, MessageParams]
        if isinstance(_params, Unset):
            params = UNSET
        else:
            params = MessageParams.from_dict(_params)

        message = cls(
            role=role,
            content=content,
            template_id=template_id,
            params=params,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Dict, Optional, Union

import httpx
from typing_extensions import Any, Unpack

from ... import errors
from ...models.fine_tuning_job_details_request import FineTuningJobDetailsRequest
from ...models.fine_tuning_job_response import FineTuningJobResponse
from ...models.v1_finetuning_job_details_create_response_404 import V1FinetuningJobDetailsCreateResponse404

# from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    **body: Unpack[FineTuningJobDetailsRequest],
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/v1/finetuning_job_details",
    }

    _json_body = body

    _kwargs["json"] = _json_body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client, response: httpx.Response
) -> Optional[Union[FineTuningJobResponse, V1FinetuningJobDetailsCreateResponse404]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = FineTuningJobResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client, response: httpx.Response
) -> Response[Union[FineTuningJobResponse, V1FinetuningJobDetailsCreateResponse404]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_finetuning_job_details_create_wrapper(client):
    def v1_finetuning_job_details_create_wrapped(
        **body: Unpack[FineTuningJobDetailsRequest],
    ) -> Union[FineTuningJobResponse, V1FinetuningJobDetailsCreateResponse404]:
        """Get details for a fine-tuning job

        Args:
            body (FineTuningJobDetailsRequest):
            body (FineTuningJobDetailsRequest):
            body (FineTuningJobDetailsRequest):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[Union[FineTuningJobResponse, V1FinetuningJobDetailsCreateResponse404]]
        """

        kwargs = _get_kwargs(
            **body,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return v1_finetuning_job_details_create_wrapped



from http import HTTPStatus
from typing import Dict, Optional

import httpx
from typing_extensions import Any, Unpack

from ... import errors
from ...models.document_input import DocumentInput
from ...models.document_output import DocumentOutput

# from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    repository_id: int,
    **body: Unpack[DocumentInput],
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": f"/api/repositories/repository/{repository_id}/document",
    }

    _body = body.to_multipart()

    _kwargs["files"] = _body

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client, response: httpx.Response) -> Optional[DocumentOutput]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = DocumentOutput.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client, response: httpx.Response) -> Response[DocumentOutput]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def api_repositories_repository_document_create_wrapper(client):
    def api_repositories_repository_document_create_wrapped(
        repository_id: int,
        **body: Unpack[DocumentInput],
    ) -> DocumentOutput:
        """
        Args:
            repository_id (int):
            body (DocumentInput):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[DocumentOutput]
        """

        kwargs = _get_kwargs(
            repository_id=repository_id,
            **body,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return api_repositories_repository_document_create_wrapped



import json
from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.datapoint import Datapoint
from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters
from ..models.fine_tuning_job_synthetic_datageneration_parameters import FineTuningJobSyntheticDatagenerationParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningJobCreate")


class FineTuningJobCreateDict(TypedDict):
    project_id: int
    base_model: str
    datapoints: List["Datapoint"]
    hyperparameters: "FineTuningJobHyperparameters"
    synthetic_datageneration_parameters: "FineTuningJobSyntheticDatagenerationParameters"
    name: Union[Unset, str] = ""
    pass


@_attrs_define
class FineTuningJobCreate:
    """
    Attributes:
        project_id (int): The id of the project to use for finetuning
        base_model (str): The slug of the base model to use for finetuning
        datapoints (List['Datapoint']): The datapoints to use for finetuning
        hyperparameters (FineTuningJobHyperparameters):
        synthetic_datageneration_parameters (FineTuningJobSyntheticDatagenerationParameters):
        name (Union[Unset, str]): Name of the fine-tuning job Default: ''.
    """

    project_id: int
    base_model: str
    datapoints: List["Datapoint"]
    hyperparameters: "FineTuningJobHyperparameters"
    synthetic_datageneration_parameters: "FineTuningJobSyntheticDatagenerationParameters"
    name: Union[Unset, str] = ""

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id

        base_model = self.base_model

        datapoints = []
        for datapoints_item_data in self.datapoints:
            datapoints_item = datapoints_item_data.to_dict()
            datapoints.append(datapoints_item)

        hyperparameters = self.hyperparameters.to_dict()

        synthetic_datageneration_parameters = self.synthetic_datageneration_parameters.to_dict()

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "base_model": base_model,
                "datapoints": datapoints,
                "hyperparameters": hyperparameters,
                "synthetic_datageneration_parameters": synthetic_datageneration_parameters,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        base_model = (
            self.base_model
            if isinstance(self.base_model, Unset)
            else (None, str(self.base_model).encode(), "text/plain")
        )

        _temp_datapoints = []
        for datapoints_item_data in self.datapoints:
            datapoints_item = datapoints_item_data.to_dict()
            _temp_datapoints.append(datapoints_item)
        datapoints = (None, json.dumps(_temp_datapoints).encode(), "application/json")

        hyperparameters = (None, json.dumps(self.hyperparameters.to_dict()).encode(), "application/json")

        synthetic_datageneration_parameters = (
            None,
            json.dumps(self.synthetic_datageneration_parameters.to_dict()).encode(),
            "application/json",
        )

        name = self.name if isinstance(self.name, Unset) else (None, str(self.name).encode(), "text/plain")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
                "base_model": base_model,
                "datapoints": datapoints,
                "hyperparameters": hyperparameters,
                "synthetic_datageneration_parameters": synthetic_datageneration_parameters,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.datapoint import Datapoint
        from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters
        from ..models.fine_tuning_job_synthetic_datageneration_parameters import (
            FineTuningJobSyntheticDatagenerationParameters,
        )

        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        base_model = d.pop("base_model")

        datapoints = []
        _datapoints = d.pop("datapoints")
        for datapoints_item_data in _datapoints:
            datapoints_item = Datapoint.from_dict(datapoints_item_data)

            datapoints.append(datapoints_item)

        hyperparameters = FineTuningJobHyperparameters.from_dict(d.pop("hyperparameters"))

        synthetic_datageneration_parameters = FineTuningJobSyntheticDatagenerationParameters.from_dict(
            d.pop("synthetic_datageneration_parameters")
        )

        name = d.pop("name", UNSET)

        fine_tuning_job_create = cls(
            project_id=project_id,
            base_model=base_model,
            datapoints=datapoints,
            hyperparameters=hyperparameters,
            synthetic_datageneration_parameters=synthetic_datageneration_parameters,
            name=name,
        )

        fine_tuning_job_create.additional_properties = d
        return fine_tuning_job_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Dict, List, Optional, Union

import httpx
from typing_extensions import Any

from ... import errors
from ...models.trace_list import TraceList
from ...models.v1_traces_list_admin_filter import V1TracesListAdminFilter
from ...models.v1_traces_list_date_filter import V1TracesListDateFilter

# from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: int,
    admin_filter: Union[Unset, V1TracesListAdminFilter] = UNSET,
    date_filter: Union[Unset, V1TracesListDateFilter] = UNSET,
    from_date: Union[Unset, str] = UNSET,
    search: Union[Unset, str] = UNSET,
    sort: Union[Unset, str] = UNSET,
    to_date: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_admin_filter: Union[Unset, str] = UNSET
    if not isinstance(admin_filter, Unset):
        json_admin_filter = admin_filter.value

    params["admin_filter"] = json_admin_filter

    json_date_filter: Union[Unset, str] = UNSET
    if not isinstance(date_filter, Unset):
        json_date_filter = date_filter.value

    params["date_filter"] = json_date_filter

    params["from_date"] = from_date

    params["project_id"] = project_id

    params["search"] = search

    params["sort"] = sort

    params["to_date"] = to_date

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/traces/",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client, response: httpx.Response) -> Optional[List["TraceList"]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = TraceList.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client, response: httpx.Response) -> Response[List["TraceList"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_traces_list_wrapper(client):
    def v1_traces_list_wrapped(
        project_id: int,
        admin_filter: Union[Unset, V1TracesListAdminFilter] = UNSET,
        date_filter: Union[Unset, V1TracesListDateFilter] = UNSET,
        from_date: Union[Unset, str] = UNSET,
        search: Union[Unset, str] = UNSET,
        sort: Union[Unset, str] = UNSET,
        to_date: Union[Unset, str] = UNSET,
    ) -> List["TraceList"]:
        """
        Args:
            admin_filter (Union[Unset, V1TracesListAdminFilter]):
            date_filter (Union[Unset, V1TracesListDateFilter]):
            from_date (Union[Unset, str]):
            project_id (int):
            search (Union[Unset, str]):
            sort (Union[Unset, str]):
            to_date (Union[Unset, str]):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[List['TraceList']]
        """

        kwargs = _get_kwargs(
            admin_filter=admin_filter,
            date_filter=date_filter,
            from_date=from_date,
            project_id=project_id,
            search=search,
            sort=sort,
            to_date=to_date,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return v1_traces_list_wrapped



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="V1FinetuningJobDetailsCreateResponse404")


class V1FinetuningJobDetailsCreateResponse404Dict(TypedDict):
    error: NotRequired[Union[Unset, str]]
    pass


@_attrs_define
class V1FinetuningJobDetailsCreateResponse404:
    """
    Attributes:
        error (Union[Unset, str]):
    """

    error: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        error = self.error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        error = d.pop("error", UNSET)

        v1_finetuning_job_details_create_response_404 = cls(
            error=error,
        )

        v1_finetuning_job_details_create_response_404.additional_properties = d
        return v1_finetuning_job_details_create_response_404

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.datapoint import Datapoint
from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters
from ..models.fine_tuning_job_response_error import FineTuningJobResponseError
from ..models.fine_tuning_job_response_evaluation_scores import FineTuningJobResponseEvaluationScores
from ..models.fine_tuning_job_synthetic_datageneration_parameters import FineTuningJobSyntheticDatagenerationParameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningJobResponse")


class FineTuningJobResponseDict(TypedDict):
    name: str
    base_model: str
    status: str
    original_dataset: List["Datapoint"]
    augmented_dataset: List["Datapoint"]
    num_topics: int
    num_augmented_datapoints: int
    created_at: datetime.datetime
    finetuned_model_id: int
    id: int
    provider_job_id: str
    hyperparameters: "FineTuningJobHyperparameters"
    synthetic_datageneration_parameters: "FineTuningJobSyntheticDatagenerationParameters"
    evaluation_scores: NotRequired[Union[Unset, FineTuningJobResponseEvaluationScores]]
    error: NotRequired[Union[Unset, FineTuningJobResponseError]]
    pass


@_attrs_define
class FineTuningJobResponse:
    """
    Attributes:
        name (str):
        base_model (str):
        status (str):
        original_dataset (List['Datapoint']):
        augmented_dataset (List['Datapoint']):
        num_topics (int):
        num_augmented_datapoints (int):
        created_at (datetime.datetime):
        finetuned_model_id (int):
        id (int):
        provider_job_id (str):
        hyperparameters (FineTuningJobHyperparameters):
        synthetic_datageneration_parameters (FineTuningJobSyntheticDatagenerationParameters):
        evaluation_scores (Union[Unset, FineTuningJobResponseEvaluationScores]):
        error (Union[Unset, FineTuningJobResponseError]):
    """

    name: str
    base_model: str
    status: str
    original_dataset: List["Datapoint"]
    augmented_dataset: List["Datapoint"]
    num_topics: int
    num_augmented_datapoints: int
    created_at: datetime.datetime
    finetuned_model_id: int
    id: int
    provider_job_id: str
    hyperparameters: "FineTuningJobHyperparameters"
    synthetic_datageneration_parameters: "FineTuningJobSyntheticDatagenerationParameters"
    evaluation_scores: Union[Unset, "FineTuningJobResponseEvaluationScores"] = UNSET
    error: Union[Unset, "FineTuningJobResponseError"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        base_model = self.base_model

        status = self.status

        original_dataset = []
        for original_dataset_item_data in self.original_dataset:
            original_dataset_item = original_dataset_item_data.to_dict()
            original_dataset.append(original_dataset_item)

        augmented_dataset = []
        for augmented_dataset_item_data in self.augmented_dataset:
            augmented_dataset_item = augmented_dataset_item_data.to_dict()
            augmented_dataset.append(augmented_dataset_item)

        num_topics = self.num_topics

        num_augmented_datapoints = self.num_augmented_datapoints

        created_at = self.created_at.isoformat()

        finetuned_model_id = self.finetuned_model_id

        id = self.id

        provider_job_id = self.provider_job_id

        hyperparameters = self.hyperparameters.to_dict()

        synthetic_datageneration_parameters = self.synthetic_datageneration_parameters.to_dict()

        evaluation_scores: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.evaluation_scores, Unset):
            evaluation_scores = self.evaluation_scores.to_dict()

        error: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.error, Unset):
            error = self.error.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "baseModel": base_model,
                "status": status,
                "originalDataset": original_dataset,
                "augmentedDataset": augmented_dataset,
                "numTopics": num_topics,
                "numAugmentedDatapoints": num_augmented_datapoints,
                "createdAt": created_at,
                "finetunedModelId": finetuned_model_id,
                "id": id,
                "providerJobId": provider_job_id,
                "hyperparameters": hyperparameters,
                "syntheticDatagenerationParameters": synthetic_datageneration_parameters,
            }
        )
        if evaluation_scores is not UNSET:
            field_dict["evaluationScores"] = evaluation_scores
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.datapoint import Datapoint
        from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters
        from ..models.fine_tuning_job_response_error import FineTuningJobResponseError
        from ..models.fine_tuning_job_response_evaluation_scores import FineTuningJobResponseEvaluationScores
        from ..models.fine_tuning_job_synthetic_datageneration_parameters import (
            FineTuningJobSyntheticDatagenerationParameters,
        )

        d = src_dict.copy() if src_dict else {}
        name = d.pop("name")

        base_model = d.pop("baseModel")

        status = d.pop("status")

        original_dataset = []
        _original_dataset = d.pop("originalDataset")
        for original_dataset_item_data in _original_dataset:
            original_dataset_item = Datapoint.from_dict(original_dataset_item_data)

            original_dataset.append(original_dataset_item)

        augmented_dataset = []
        _augmented_dataset = d.pop("augmentedDataset")
        for augmented_dataset_item_data in _augmented_dataset:
            augmented_dataset_item = Datapoint.from_dict(augmented_dataset_item_data)

            augmented_dataset.append(augmented_dataset_item)

        num_topics = d.pop("numTopics")

        num_augmented_datapoints = d.pop("numAugmentedDatapoints")

        created_at = isoparse(d.pop("createdAt"))

        finetuned_model_id = d.pop("finetunedModelId")

        id = d.pop("id")

        provider_job_id = d.pop("providerJobId")

        hyperparameters = FineTuningJobHyperparameters.from_dict(d.pop("hyperparameters"))

        synthetic_datageneration_parameters = FineTuningJobSyntheticDatagenerationParameters.from_dict(
            d.pop("syntheticDatagenerationParameters")
        )

        _evaluation_scores = d.pop("evaluationScores", UNSET)
        evaluation_scores: Union[Unset, FineTuningJobResponseEvaluationScores]
        if isinstance(_evaluation_scores, Unset):
            evaluation_scores = UNSET
        else:
            evaluation_scores = FineTuningJobResponseEvaluationScores.from_dict(_evaluation_scores)

        _error = d.pop("error", UNSET)
        error: Union[Unset, FineTuningJobResponseError]
        if isinstance(_error, Unset):
            error = UNSET
        else:
            error = FineTuningJobResponseError.from_dict(_error)

        fine_tuning_job_response = cls(
            name=name,
            base_model=base_model,
            status=status,
            original_dataset=original_dataset,
            augmented_dataset=augmented_dataset,
            num_topics=num_topics,
            num_augmented_datapoints=num_augmented_datapoints,
            created_at=created_at,
            finetuned_model_id=finetuned_model_id,
            id=id,
            provider_job_id=provider_job_id,
            hyperparameters=hyperparameters,
            synthetic_datageneration_parameters=synthetic_datageneration_parameters,
            evaluation_scores=evaluation_scores,
            error=error,
        )

        fine_tuning_job_response.additional_properties = d
        return fine_tuning_job_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.validation_detail_error_messages_item import ValidationDetailErrorMessagesItem

T = TypeVar("T", bound="ValidationDetail")


class ValidationDetailDict(TypedDict):
    error_messages: List["ValidationDetailErrorMessagesItem"]
    pass


@_attrs_define
class ValidationDetail:
    """
    Attributes:
        error_messages (List['ValidationDetailErrorMessagesItem']): Error messages for the field.
    """

    error_messages: List["ValidationDetailErrorMessagesItem"]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        error_messages = []
        for error_messages_item_data in self.error_messages:
            error_messages_item = error_messages_item_data.to_dict()
            error_messages.append(error_messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "error_messages": error_messages,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.validation_detail_error_messages_item import ValidationDetailErrorMessagesItem

        d = src_dict.copy() if src_dict else {}
        error_messages = []
        _error_messages = d.pop("error_messages")
        for error_messages_item_data in _error_messages:
            error_messages_item = ValidationDetailErrorMessagesItem.from_dict(error_messages_item_data)

            error_messages.append(error_messages_item)

        validation_detail = cls(
            error_messages=error_messages,
        )

        validation_detail.additional_properties = d
        return validation_detail

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.fine_tuning_job_response import FineTuningJobResponse

T = TypeVar("T", bound="InitPageDataResponse")


class InitPageDataResponseDict(TypedDict):
    ft_jobs: List["FineTuningJobResponse"]
    pass


@_attrs_define
class InitPageDataResponse:
    """
    Attributes:
        ft_jobs (List['FineTuningJobResponse']):
    """

    ft_jobs: List["FineTuningJobResponse"]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ft_jobs = []
        for ft_jobs_item_data in self.ft_jobs:
            ft_jobs_item = ft_jobs_item_data.to_dict()
            ft_jobs.append(ft_jobs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ftJobs": ft_jobs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.fine_tuning_job_response import FineTuningJobResponse

        d = src_dict.copy() if src_dict else {}
        ft_jobs = []
        _ft_jobs = d.pop("ftJobs")
        for ft_jobs_item_data in _ft_jobs:
            ft_jobs_item = FineTuningJobResponse.from_dict(ft_jobs_item_data)

            ft_jobs.append(ft_jobs_item)

        init_page_data_response = cls(
            ft_jobs=ft_jobs,
        )

        init_page_data_response.additional_properties = d
        return init_page_data_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Dict, Optional

import httpx
from typing_extensions import Any, Unpack

from ... import errors
from ...models.init_page_data_request import InitPageDataRequest
from ...models.init_page_data_response import InitPageDataResponse

# from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    **body: Unpack[InitPageDataRequest],
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/v1/init_page_data",
    }

    _json_body = body

    _kwargs["json"] = _json_body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client, response: httpx.Response) -> Optional[InitPageDataResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = InitPageDataResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client, response: httpx.Response) -> Response[InitPageDataResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def v1_init_page_data_create_wrapper(client):
    def v1_init_page_data_create_wrapped(
        **body: Unpack[InitPageDataRequest],
    ) -> InitPageDataResponse:
        """Get initialization data for a project page

        Args:
            body (InitPageDataRequest):
            body (InitPageDataRequest):
            body (InitPageDataRequest):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[InitPageDataResponse]
        """

        kwargs = _get_kwargs(
            **body,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return v1_init_page_data_create_wrapped



from typing import Dict, List, Type, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.parameters_properties import ParametersProperties

T = TypeVar("T", bound="Parameters")


class ParametersDict(TypedDict):
    type: str
    properties: "ParametersProperties"
    required: List[str]
    pass


@_attrs_define
class Parameters:
    """
    Attributes:
        type (str):
        properties (ParametersProperties):
        required (List[str]):
    """

    type: str
    properties: "ParametersProperties"
    required: List[str]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        properties = self.properties.to_dict()

        required = self.required

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "properties": properties,
                "required": required,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.parameters_properties import ParametersProperties

        d = src_dict.copy() if src_dict else {}
        type = d.pop("type")

        properties = ParametersProperties.from_dict(d.pop("properties"))

        required = cast(List[str], d.pop("required"))

        parameters = cls(
            type=type,
            properties=properties,
            required=required,
        )

        parameters.additional_properties = d
        return parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="Enhancement")


class EnhancementDict(TypedDict):
    ids: NotRequired[Union[Unset, List[int]]]
    limit: NotRequired[Union[Unset, int]]
    similarity_threshold: NotRequired[Union[Unset, float]]
    pass


@_attrs_define
class Enhancement:
    """
    Attributes:
        ids (Union[Unset, List[int]]): The IDs of the repositories to use.
        limit (Union[Unset, int]):
        similarity_threshold (Union[Unset, float]):
    """

    ids: Union[Unset, List[int]] = UNSET
    limit: Union[Unset, int] = UNSET
    similarity_threshold: Union[Unset, float] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ids: Union[Unset, List[int]] = UNSET
        if not isinstance(self.ids, Unset):
            ids = self.ids

        limit = self.limit

        similarity_threshold = self.similarity_threshold

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ids is not UNSET:
            field_dict["ids"] = ids
        if limit is not UNSET:
            field_dict["limit"] = limit
        if similarity_threshold is not UNSET:
            field_dict["similarity_threshold"] = similarity_threshold

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        ids = cast(List[int], d.pop("ids", UNSET))

        limit = d.pop("limit", UNSET)

        similarity_threshold = d.pop("similarity_threshold", UNSET)

        enhancement = cls(
            ids=ids,
            limit=limit,
            similarity_threshold=similarity_threshold,
        )

        enhancement.additional_properties = d
        return enhancement

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.function import Function
from ..models.type_enum import TypeEnum

T = TypeVar("T", bound="Tool")


class ToolDict(TypedDict):
    type: TypeEnum
    function: "Function"
    pass


@_attrs_define
class Tool:
    """
    Attributes:
        type (TypeEnum): * `function` - function
        function (Function):
    """

    type: TypeEnum
    function: "Function"

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        function = self.function.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "function": function,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.function import Function

        d = src_dict.copy() if src_dict else {}
        type = TypeEnum(d.pop("type"))

        function = Function.from_dict(d.pop("function"))

        tool = cls(
            type=type,
            function=function,
        )

        tool.additional_properties = d
        return tool

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class V1TracesListDateFilter(str, Enum):
    LAST_DAY = "last_day"
    LAST_HOUR = "last_hour"
    LAST_MONTH = "last_month"
    LAST_WEEK = "last_week"
    LAST_YEAR = "last_year"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.document_chunks import DocumentChunks
from ..models.response_choice import ResponseChoice
from ..models.usage import Usage
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatCompletionResponse")


class ChatCompletionResponseDict(TypedDict):
    choices: List["ResponseChoice"]
    created: int
    model: str
    provider_name: str
    provider_id: str
    usage: "Usage"
    trace_id: str
    document_chunks: NotRequired[Union[Unset, List["DocumentChunks"]]]
    pass


@_attrs_define
class ChatCompletionResponse:
    """
    Attributes:
        choices (List['ResponseChoice']): A list of chat completion choices. Can be more than one if n is greater than
            1.
        created (int): The Unix timestamp (in seconds) of when the chat completion was created. Each chunk has the same
            timestamp.
        model (str): The model to generate the completion.
        provider_name (str): The name of the provider that generated the completion.
        provider_id (str): The ID of the provider that generated the completion.
        usage (Usage):
        trace_id (str): The trace ID of the completion.
        document_chunks (Union[Unset, List['DocumentChunks']]): Chunks used to improve the completion
    """

    choices: List["ResponseChoice"]
    created: int
    model: str
    provider_name: str
    provider_id: str
    usage: "Usage"
    trace_id: str
    document_chunks: Union[Unset, List["DocumentChunks"]] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        choices = []
        for choices_item_data in self.choices:
            choices_item = choices_item_data.to_dict()
            choices.append(choices_item)

        created = self.created

        model = self.model

        provider_name = self.provider_name

        provider_id = self.provider_id

        usage = self.usage.to_dict()

        trace_id = self.trace_id

        document_chunks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.document_chunks, Unset):
            document_chunks = []
            for document_chunks_item_data in self.document_chunks:
                document_chunks_item = document_chunks_item_data.to_dict()
                document_chunks.append(document_chunks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "choices": choices,
                "created": created,
                "model": model,
                "provider_name": provider_name,
                "provider_id": provider_id,
                "usage": usage,
                "trace_id": trace_id,
            }
        )
        if document_chunks is not UNSET:
            field_dict["document_chunks"] = document_chunks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.document_chunks import DocumentChunks
        from ..models.response_choice import ResponseChoice
        from ..models.usage import Usage

        d = src_dict.copy() if src_dict else {}
        choices = []
        _choices = d.pop("choices")
        for choices_item_data in _choices:
            choices_item = ResponseChoice.from_dict(choices_item_data)

            choices.append(choices_item)

        created = d.pop("created")

        model = d.pop("model")

        provider_name = d.pop("provider_name")

        provider_id = d.pop("provider_id")

        usage = Usage.from_dict(d.pop("usage"))

        trace_id = d.pop("trace_id")

        document_chunks = []
        _document_chunks = d.pop("document_chunks", UNSET)
        for document_chunks_item_data in _document_chunks or []:
            document_chunks_item = DocumentChunks.from_dict(document_chunks_item_data)

            document_chunks.append(document_chunks_item)

        chat_completion_response = cls(
            choices=choices,
            created=created,
            model=model,
            provider_name=provider_name,
            provider_id=provider_id,
            usage=usage,
            trace_id=trace_id,
            document_chunks=document_chunks,
        )

        chat_completion_response.additional_properties = d
        return chat_completion_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

T = TypeVar("T", bound="FineTuningJobHyperparameters")


class FineTuningJobHyperparametersDict(TypedDict):
    batch_size: int
    learning_rate_multiplier: float
    num_epochs: int
    pass


@_attrs_define
class FineTuningJobHyperparameters:
    """
    Attributes:
        batch_size (int): Batch size for fine-tuning
        learning_rate_multiplier (float): Scaling factor for the learning rate. A smaller learning rate may be useful to
            avoid overfitting.
        num_epochs (int): Number of epochs for fine-tuning
    """

    batch_size: int
    learning_rate_multiplier: float
    num_epochs: int

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        batch_size = self.batch_size

        learning_rate_multiplier = self.learning_rate_multiplier

        num_epochs = self.num_epochs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "batch_size": batch_size,
                "learning_rate_multiplier": learning_rate_multiplier,
                "num_epochs": num_epochs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        batch_size = d.pop("batch_size")

        learning_rate_multiplier = d.pop("learning_rate_multiplier")

        num_epochs = d.pop("num_epochs")

        fine_tuning_job_hyperparameters = cls(
            batch_size=batch_size,
            learning_rate_multiplier=learning_rate_multiplier,
            num_epochs=num_epochs,
        )

        fine_tuning_job_hyperparameters.additional_properties = d
        return fine_tuning_job_hyperparameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.output_function import OutputFunction

T = TypeVar("T", bound="ToolCall")


class ToolCallDict(TypedDict):
    id: str
    function: "OutputFunction"
    type: str
    pass


@_attrs_define
class ToolCall:
    """
    Attributes:
        id (str): The ID of the tool call.
        function (OutputFunction):
        type (str): The type of tool call.
    """

    id: str
    function: "OutputFunction"
    type: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        function = self.function.to_dict()

        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "function": function,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.output_function import OutputFunction

        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        function = OutputFunction.from_dict(d.pop("function"))

        type = d.pop("type")

        tool_call = cls(
            id=id,
            function=function,
            type=type,
        )

        tool_call.additional_properties = d
        return tool_call

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.messages_role_enum import MessagesRoleEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="Messages")


class MessagesDict(TypedDict):
    role: NotRequired[Union[Unset, MessagesRoleEnum]]
    content: NotRequired[Union[Unset, str]]
    pass


@_attrs_define
class Messages:
    """
    Attributes:
        role (Union[Unset, MessagesRoleEnum]): * `user` - user
            * `assistant` - assistant
            * `system` - system
        content (Union[Unset, str]):
    """

    role: Union[Unset, MessagesRoleEnum] = UNSET
    content: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        content = self.content

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if role is not UNSET:
            field_dict["role"] = role
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        _role = d.pop("role", UNSET)
        role: Union[Unset, MessagesRoleEnum]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = MessagesRoleEnum(_role)

        content = d.pop("content", UNSET)

        messages = cls(
            role=role,
            content=content,
        )

        messages.additional_properties = d
        return messages

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ModelProviderEnum(str, Enum):
    ANTHROPIC = "anthropic"
    AZURE = "azure"
    GROQ = "groq"
    OPENAI = "openai"
    PREM = "prem"
    PROXY_PREM_AI = "proxy-prem-ai"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.message_content_type_1_item_image_url import MessageContentType1ItemImageUrl
from ..models.message_content_type_1_item_type import MessageContentType1ItemType
from ..types import UNSET, Unset

T = TypeVar("T", bound="MessageContentType1Item")


class MessageContentType1ItemDict(TypedDict):
    type: MessageContentType1ItemType
    text: NotRequired[Union[Unset, str]]
    image_url: NotRequired[Union[Unset, MessageContentType1ItemImageUrl]]
    pass


@_attrs_define
class MessageContentType1Item:
    """
    Attributes:
        type (MessageContentType1ItemType):
        text (Union[Unset, str]):
        image_url (Union[Unset, MessageContentType1ItemImageUrl]):
    """

    type: MessageContentType1ItemType
    text: Union[Unset, str] = UNSET
    image_url: Union[Unset, "MessageContentType1ItemImageUrl"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        text = self.text

        image_url: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.image_url, Unset):
            image_url = self.image_url.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if text is not UNSET:
            field_dict["text"] = text
        if image_url is not UNSET:
            field_dict["image_url"] = image_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_content_type_1_item_image_url import MessageContentType1ItemImageUrl

        d = src_dict.copy() if src_dict else {}
        type = MessageContentType1ItemType(d.pop("type"))

        text = d.pop("text", UNSET)

        _image_url = d.pop("image_url", UNSET)
        image_url: Union[Unset, MessageContentType1ItemImageUrl]
        if isinstance(_image_url, Unset):
            image_url = UNSET
        else:
            image_url = MessageContentType1ItemImageUrl.from_dict(_image_url)

        message_content_type_1_item = cls(
            type=type,
            text=text,
            image_url=image_url,
        )

        message_content_type_1_item.additional_properties = d
        return message_content_type_1_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.messages import Messages
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeedbackCreateFeedback")


class FeedbackCreateFeedbackDict(TypedDict):
    positive: Union[Unset, bool] = False
    messages: NotRequired[Union[Unset, List["Messages"]]]
    pass


@_attrs_define
class FeedbackCreateFeedback:
    """
    Attributes:
        positive (Union[Unset, bool]):  Default: False.
        messages (Union[Unset, List['Messages']]):
    """

    positive: Union[Unset, bool] = False
    messages: Union[Unset, List["Messages"]] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        positive = self.positive

        messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.messages, Unset):
            messages = []
            for messages_item_data in self.messages:
                messages_item = messages_item_data.to_dict()
                messages.append(messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if positive is not UNSET:
            field_dict["positive"] = positive
        if messages is not UNSET:
            field_dict["messages"] = messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.messages import Messages

        d = src_dict.copy() if src_dict else {}
        positive = d.pop("positive", UNSET)

        messages = []
        _messages = d.pop("messages", UNSET)
        for messages_item_data in _messages or []:
            messages_item = Messages.from_dict(messages_item_data)

            messages.append(messages_item)

        feedback_create_feedback = cls(
            positive=positive,
            messages=messages,
        )

        feedback_create_feedback.additional_properties = d
        return feedback_create_feedback

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

T = TypeVar("T", bound="FineTuningJobSyntheticDatagenerationParameters")


class FineTuningJobSyntheticDatagenerationParametersDict(TypedDict):
    run_synthetic_datageneration: bool
    min_num_datapoints_for_ft: int
    temperature: float
    positive_instructions: str
    negative_instructions: str
    pass


@_attrs_define
class FineTuningJobSyntheticDatagenerationParameters:
    """
    Attributes:
        run_synthetic_datageneration (bool): Whether to run synthetic datageneration
        min_num_datapoints_for_ft (int): Minimum number of datapoints required for fine-tuning
        temperature (float): Temperature for synthetic datageneration
        positive_instructions (str): Positive instructions for synthetic datageneration, what the datapoints should be
            about
        negative_instructions (str): Negative instructions for synthetic datageneration, what the datapoints should not
            be about
    """

    run_synthetic_datageneration: bool
    min_num_datapoints_for_ft: int
    temperature: float
    positive_instructions: str
    negative_instructions: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        run_synthetic_datageneration = self.run_synthetic_datageneration

        min_num_datapoints_for_ft = self.min_num_datapoints_for_ft

        temperature = self.temperature

        positive_instructions = self.positive_instructions

        negative_instructions = self.negative_instructions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "run_synthetic_datageneration": run_synthetic_datageneration,
                "min_num_datapoints_for_ft": min_num_datapoints_for_ft,
                "temperature": temperature,
                "positive_instructions": positive_instructions,
                "negative_instructions": negative_instructions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        run_synthetic_datageneration = d.pop("run_synthetic_datageneration")

        min_num_datapoints_for_ft = d.pop("min_num_datapoints_for_ft")

        temperature = d.pop("temperature")

        positive_instructions = d.pop("positive_instructions")

        negative_instructions = d.pop("negative_instructions")

        fine_tuning_job_synthetic_datageneration_parameters = cls(
            run_synthetic_datageneration=run_synthetic_datageneration,
            min_num_datapoints_for_ft=min_num_datapoints_for_ft,
            temperature=temperature,
            positive_instructions=positive_instructions,
            negative_instructions=negative_instructions,
        )

        fine_tuning_job_synthetic_datageneration_parameters.additional_properties = d
        return fine_tuning_job_synthetic_datageneration_parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Dict, List, Tuple, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0
from ..models.enhancement import Enhancement
from ..models.message import Message
from ..models.tool import Tool
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatCompletionInput")


class ChatCompletionInputDict(TypedDict):
    project_id: int
    messages: List["Message"]
    session_id: NotRequired[Union[Unset, str]]
    repositories: NotRequired[Union[Unset, Enhancement]]
    model: NotRequired[Union[Unset, str]]
    system_prompt: NotRequired[Union[Unset, str]]
    max_tokens: NotRequired[Union[None, Unset, int]]
    stream: NotRequired[Union[Unset, bool]]
    temperature: Union[Unset, float] = 1.0
    tools: NotRequired[Union[List["Tool"], None, Unset]]
    response_format: NotRequired[Union["ChatCompletionInputResponseFormatType0", None, Unset]]
    pass


@_attrs_define
class ChatCompletionInput:
    """
    Attributes:
        project_id (int): The ID of the project to use.
        messages (List['Message']): A list of messages comprising the conversation so far.
        session_id (Union[Unset, str]): The ID of the session to use. It helps to track the chat history.
        repositories (Union[Unset, Enhancement]):
        model (Union[Unset, str]): ID of the model to use. See the model endpoint compatibility table for details.
        system_prompt (Union[Unset, str]): The system prompt to use.
        max_tokens (Union[None, Unset, int]): The maximum number of tokens to generate in the chat completion.
        stream (Union[Unset, bool]): If set, partial message deltas will be sent, like in ChatGPT.
        temperature (Union[Unset, float]): What sampling temperature to use, between 0 and 2. Default: 1.0.
        tools (Union[List['Tool'], None, Unset]): The tools to use in the completion.
        response_format (Union['ChatCompletionInputResponseFormatType0', None, Unset]): The format of the response. Can
            be a JSON schema or a simple `json_object` type
    """

    project_id: int
    messages: List["Message"]
    session_id: Union[Unset, str] = UNSET
    repositories: Union[Unset, "Enhancement"] = UNSET
    model: Union[Unset, str] = UNSET
    system_prompt: Union[Unset, str] = UNSET
    max_tokens: Union[None, Unset, int] = UNSET
    stream: Union[Unset, bool] = UNSET
    temperature: Union[Unset, float] = 1.0
    tools: Union[List["Tool"], None, Unset] = UNSET
    response_format: Union["ChatCompletionInputResponseFormatType0", None, Unset] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0

        project_id = self.project_id

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        session_id = self.session_id

        repositories: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.repositories, Unset):
            repositories = self.repositories.to_dict()

        model = self.model

        system_prompt = self.system_prompt

        max_tokens: Union[None, Unset, int]
        if isinstance(self.max_tokens, Unset):
            max_tokens = UNSET
        else:
            max_tokens = self.max_tokens

        stream = self.stream

        temperature = self.temperature

        tools: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.tools, Unset):
            tools = UNSET
        elif isinstance(self.tools, list):
            tools = []
            for tools_type_0_item_data in self.tools:
                tools_type_0_item = tools_type_0_item_data.to_dict()
                tools.append(tools_type_0_item)

        else:
            tools = self.tools

        response_format: Union[Dict[str, Any], None, Unset]
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ChatCompletionInputResponseFormatType0):
            response_format = self.response_format.to_dict()
        else:
            response_format = self.response_format

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "messages": messages,
            }
        )
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if repositories is not UNSET:
            field_dict["repositories"] = repositories
        if model is not UNSET:
            field_dict["model"] = model
        if system_prompt is not UNSET:
            field_dict["system_prompt"] = system_prompt
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if stream is not UNSET:
            field_dict["stream"] = stream
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if tools is not UNSET:
            field_dict["tools"] = tools
        if response_format is not UNSET:
            field_dict["response_format"] = response_format

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        _temp_messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            _temp_messages.append(messages_item)
        messages = (None, json.dumps(_temp_messages).encode(), "application/json")

        session_id = (
            self.session_id
            if isinstance(self.session_id, Unset)
            else (None, str(self.session_id).encode(), "text/plain")
        )

        repositories: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.repositories, Unset):
            repositories = (None, json.dumps(self.repositories.to_dict()).encode(), "application/json")

        model = self.model if isinstance(self.model, Unset) else (None, str(self.model).encode(), "text/plain")

        system_prompt = (
            self.system_prompt
            if isinstance(self.system_prompt, Unset)
            else (None, str(self.system_prompt).encode(), "text/plain")
        )

        max_tokens: Union[None, Unset, int]
        if isinstance(self.max_tokens, Unset):
            max_tokens = UNSET
        else:
            max_tokens = self.max_tokens

        stream = self.stream if isinstance(self.stream, Unset) else (None, str(self.stream).encode(), "text/plain")

        temperature = (
            self.temperature
            if isinstance(self.temperature, Unset)
            else (None, str(self.temperature).encode(), "text/plain")
        )

        tools: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.tools, Unset):
            tools = UNSET
        elif isinstance(self.tools, list):
            _temp_tools = []
            for tools_type_0_item_data in self.tools:
                tools_type_0_item = tools_type_0_item_data.to_dict()
                _temp_tools.append(tools_type_0_item)
            tools = (None, json.dumps(_temp_tools).encode(), "application/json")

        else:
            tools = self.tools

        response_format: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, ChatCompletionInputResponseFormatType0):
            response_format = (None, json.dumps(self.response_format.to_dict()).encode(), "application/json")
        else:
            response_format = self.response_format

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
                "messages": messages,
            }
        )
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if repositories is not UNSET:
            field_dict["repositories"] = repositories
        if model is not UNSET:
            field_dict["model"] = model
        if system_prompt is not UNSET:
            field_dict["system_prompt"] = system_prompt
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if stream is not UNSET:
            field_dict["stream"] = stream
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if tools is not UNSET:
            field_dict["tools"] = tools
        if response_format is not UNSET:
            field_dict["response_format"] = response_format

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_completion_input_response_format_type_0 import ChatCompletionInputResponseFormatType0
        from ..models.enhancement import Enhancement
        from ..models.message import Message
        from ..models.tool import Tool

        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = Message.from_dict(messages_item_data)

            messages.append(messages_item)

        session_id = d.pop("session_id", UNSET)

        _repositories = d.pop("repositories", UNSET)
        repositories: Union[Unset, Enhancement]
        if isinstance(_repositories, Unset):
            repositories = UNSET
        else:
            repositories = Enhancement.from_dict(_repositories)

        model = d.pop("model", UNSET)

        system_prompt = d.pop("system_prompt", UNSET)

        def _parse_max_tokens(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_tokens = _parse_max_tokens(d.pop("max_tokens", UNSET))

        stream = d.pop("stream", UNSET)

        temperature = d.pop("temperature", UNSET)

        def _parse_tools(data: object) -> Union[List["Tool"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tools_type_0 = []
                _tools_type_0 = data
                for tools_type_0_item_data in _tools_type_0:
                    tools_type_0_item = Tool.from_dict(tools_type_0_item_data)

                    tools_type_0.append(tools_type_0_item)

                return tools_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["Tool"], None, Unset], data)

        tools = _parse_tools(d.pop("tools", UNSET))

        def _parse_response_format(data: object) -> Union["ChatCompletionInputResponseFormatType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_0 = ChatCompletionInputResponseFormatType0.from_dict(data)

                return response_format_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ChatCompletionInputResponseFormatType0", None, Unset], data)

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        chat_completion_input = cls(
            project_id=project_id,
            messages=messages,
            session_id=session_id,
            repositories=repositories,
            model=model,
            system_prompt=system_prompt,
            max_tokens=max_tokens,
            stream=stream,
            temperature=temperature,
            tools=tools,
            response_format=response_format,
        )

        chat_completion_input.additional_properties = d
        return chat_completion_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..types import Unset

T = TypeVar("T", bound="InitPageDataRequest")


class InitPageDataRequestDict(TypedDict):
    project_id: int
    pass


@_attrs_define
class InitPageDataRequest:
    """
    Attributes:
        project_id (int):
    """

    project_id: int

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
            }
        )

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        init_page_data_request = cls(
            project_id=project_id,
        )

        init_page_data_request.additional_properties = d
        return init_page_data_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="Usage")


class UsageDict(TypedDict):
    prompt_tokens: int
    total_tokens: int
    completion_tokens: NotRequired[Union[Unset, int]]
    pass


@_attrs_define
class Usage:
    """
    Attributes:
        prompt_tokens (int):
        total_tokens (int):
        completion_tokens (Union[Unset, int]):
    """

    prompt_tokens: int
    total_tokens: int
    completion_tokens: Union[Unset, int] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        prompt_tokens = self.prompt_tokens

        total_tokens = self.total_tokens

        completion_tokens = self.completion_tokens

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prompt_tokens": prompt_tokens,
                "total_tokens": total_tokens,
            }
        )
        if completion_tokens is not UNSET:
            field_dict["completion_tokens"] = completion_tokens

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        prompt_tokens = d.pop("prompt_tokens")

        total_tokens = d.pop("total_tokens")

        completion_tokens = d.pop("completion_tokens", UNSET)

        usage = cls(
            prompt_tokens=prompt_tokens,
            total_tokens=total_tokens,
            completion_tokens=completion_tokens,
        )

        usage.additional_properties = d
        return usage

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
